<?php

namespace EventPortal\Http\Controllers\Delegate\Event;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\ValueObjects\EventDetails;
use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Support\Arr;
use Inertia\Response;

class LandingController extends Controller
{
    /** @var Event */
    protected $event;

    public function __invoke(Event $event): Response
    {
        $this->event = $event;

        return DelegateRender::component('Delegate/Landing/Index')
            ->withPageHeader(
                PageHeader::make()
                    ->title($this->event->name)
                    ->description($this->event->snippet)
                    ->callToAction($this->getCallToAction())
            )
            ->withEventDetails(EventDetails::make($this->event))
            ->withSpeakers($this->event->speakers)
            ->get();
    }

    public function getCallToAction(): Button
    {
        $status = EventDelegate::forEventAndDelegate($this->event, auth()->user()->delegate)
            ->firstOrFail()
            ->status;

        $buttons = [
            'invited' => Button::make('Begin registration')
                ->method(InertiaMethod::VISIT())
                ->route('delegate.landing'),
            'registered' => Button::make('View registration details')
                ->method(InertiaMethod::VISIT())
                ->route('delegate.landing'),
        ];

        $fallbackButton = Button::make('Begin registration')
            ->method(InertiaMethod::VISIT())
            ->route('delegate.landing');

        return Arr::get($buttons, $status, $fallbackButton);
    }
}
