<?php

namespace EventPortal\Tests\Unit\Event\Form\Requests;

use EventPortal\Domain\Event\Form\Requests\UpdateFormRequest;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Validator;

class UpdateFormRequestTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function form_name_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            [],
            $request->rules()
        );

        $this->assertEquals(
            'The name field is required.',
            $validator->errors()->first('name')
        );
    }

    /** @test **/
    public function form_type_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['type' => ''],
            $request->rules()
        );

        $this->assertEquals(
            'The type field is required.',
            $validator->errors()->first('type')
        );
    }

    /** @test **/
    public function form_type_value_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['type' => ['value' => '']],
            $request->rules()
        );

        $this->assertEquals(
            'The type.value field is required.',
            $validator->errors()->first('type.value')
        );
    }

    /** @test **/
    public function form_type_should_be_valid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['type' => ['value' => 'INVALID']],
            $request->rules()
        );

        $this->assertEquals(
            'validation.enum',
            $validator->errors()->first('type.value')
        );
    }

    /** @test **/
    public function step_name_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['name' => '']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.name field is required.',
            $validator->errors()->first('steps.0.name')
        );
    }

    /** @test **/
    public function step_order_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['order' => '']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.order field is required.',
            $validator->errors()->first('steps.0.order')
        );
    }

    /** @test **/
    public function step_order_must_be_an_integer(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['order' => 'NOT-AN-INT']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.order must be an integer.',
            $validator->errors()->first('steps.0.order')
        );
    }

    /** @test **/
    public function step_uuid_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['uuid' => '']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.uuid field is required.',
            $validator->errors()->first('steps.0.uuid')
        );
    }

    /** @test **/
    public function step_uuid_must_be_a_valid_uuid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['uuid' => 'INVALID']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.uuid must be a valid UUID.',
            $validator->errors()->first('steps.0.uuid')
        );
    }

    /** @test **/
    public function section_name_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['name' => '']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.name field is required.',
            $validator->errors()->first('steps.0.sections.0.name')
        );
    }

    /** @test **/
    public function section_order_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['order' => '']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.order field is required.',
            $validator->errors()->first('steps.0.sections.0.order')
        );
    }

    /** @test **/
    public function section_order_must_be_an_integer(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['order' => 'NOT-AN-INT']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.order must be an integer.',
            $validator->errors()->first('steps.0.sections.0.order')
        );
    }

    /** @test **/
    public function section_uuid_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['uuid' => '']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.uuid field is required.',
            $validator->errors()->first('steps.0.sections.0.uuid')
        );
    }

    /** @test **/
    public function section_uuid_must_be_a_valid_uuid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['uuid' => 'INVALID']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.uuid must be a valid UUID.',
            $validator->errors()->first('steps.0.sections.0.uuid')
        );
    }

    /** @test **/
    public function question_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['question' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.question field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.question')
        );
    }

    /** @test **/
    public function question_order_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['order' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.order field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.order')
        );
    }

    /** @test **/
    public function question_order_must_be_an_integer(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['order' => 'NOT-AN-INT']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.order must be an integer.',
            $validator->errors()->first('steps.0.sections.0.questions.0.order')
        );
    }

    /** @test **/
    public function question_uuid_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['uuid' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.uuid field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.uuid')
        );
    }

    /** @test **/
    public function question_uuid_must_be_a_valid_uuid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['uuid' => 'INVALID']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.uuid must be a valid UUID.',
            $validator->errors()->first('steps.0.sections.0.questions.0.uuid')
        );
    }

    /** @test **/
    public function question_component_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['component' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.component field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.component')
        );
    }

    /** @test **/
    public function question_component_name_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['component' => ['name' => '']]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.component.name field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.component.name')
        );
    }

    /** @test **/
    public function question_component_name_value_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['component' => ['name' => ['value' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.component.name.value field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.component.name.value')
        );
    }

    /** @test **/
    public function question_type_must_be_valid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['component' => ['name' => ['value' => 'INVALID']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'validation.enum',
            $validator->errors()->first('steps.0.sections.0.questions.0.component.name.value')
        );
    }

    /** @test **/
    public function question_gdpr_type_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['gdprType' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.gdprType field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.gdprType')
        );
    }

    /** @test **/
    public function question_gdpr_type_must_be_valid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['gdprType' => 'INVALID']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'validation.enum',
            $validator->errors()->first('steps.0.sections.0.questions.0.gdprType')
        );
    }

    /** @test **/
    public function question_required_field_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [[]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.required field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.required')
        );
    }

    /** @test **/
    public function question_required_field_must_be_boolean(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['required' => 'NOT BOOLEAN']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.required field must be true or false.',
            $validator->errors()->first('steps.0.sections.0.questions.0.required')
        );
    }

    /** @test **/
    public function option_value_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['value' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.value field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.value')
        );
    }

    /** @test **/
    public function option_order_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['order' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.order field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.order')
        );
    }

    /** @test **/
    public function option_order_must_be_an_integer(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['order' => 'NOT-AN-INT']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.order must be an integer.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.order')
        );
    }

    /** @test **/
    public function option_uuid_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['uuid' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.uuid field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.uuid')
        );
    }

    /** @test **/
    public function option_uuid_must_be_a_valid_uuid(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['uuid' => 'INVALID']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.uuid must be a valid UUID.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.uuid')
        );
    }

    /** @test **/
    public function question_validation_rule_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['validations' => [['rule' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.validations.0.rule field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.validations.0.rule')
        );
    }

    /** @test **/
    public function question_validation_message_is_required(): void
    {
        $request = (new UpdateFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['validations' => [['message' => '']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.validations.0.message field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.validations.0.message')
        );
    }
}
